/*
 *	Read in makefile
 */


#include <stdio.h>
#include <ctype.h>
#include "h.h"


int		lineno;


/*
 *	Syntax error handler.  Print message, with line number, and exits.
 */
void
error(msg, a1, a2, a3)
    char	   *msg;
{
    fprintf(stderr, "%s: ", myname);
    fprintf(stderr, msg, a1, a2, a3);
    if (lineno)
	fprintf(stderr, " near line %d", lineno);
    fputc('\n', stderr);
    exit(1);
}


/*
 *	Read a line into the supplied buffer of length 'size'.  Remove
 *	comments, ignore blank lines. Deal with quoted (\) #, and
 *	quoted newlines.  If EOF return TRUE.
 *
 * -hu: "Deal with quoted #"?  What the bloody heck is that?  '#' is a
 *      comment introducer, from it to end-of-line should be ignored.
 *
 * 	A quoted # is just that. Sometimes you need to get a # character
 *	onto the command line to be executed, as is the case for example
 *	with amiga wildcards. On the Amiga, #? is what on most systems
 *	is just *, with WB2.0 * was introduced as an _optional_ synonym
 *	for #? on the Amiga. See readme.vinsci for an example.
 *					-- vinsci@soft.fi
 */
bool
getline (str, size, fd)
char *str;
int  size;
FILE  *fd;
{
    register char  *p;
    register char  *q;		/*OIS*0.80*/
    register char  *buf;
    int 	    pos = 0;

    size--;	/* disccount one byte for null terminator */

    for (;;) {

	if ((size - pos) <= 0) error ("Line too long");
	buf = str + pos;
	if (fgets(buf, size - pos, fd) == NULL)
	    return TRUE;	/* EOF	*/

	lineno++;

	if ((p = index(buf, '\n')) == NULL)
	  error("Line too long");

#if !defined(amiga)
	/* strip out comment */
	if ((p = index(buf, '#')) != NULL) {
	  if (p == buf) continue;
	  *p = '\0';
	}
#else
	/* strip out comment, but allow the # character to be escaped
	 * so it can be used as a wildcard character (the amiga uses
	 * # as one of several wildcard characters).
	 * Example:
	 *
	 *	clean:
	 *		delete \#?.o \#?.out # Delete the work and output files
	 */
	p = buf;
	while ((p = index(p, '#')) != NULL) {
	  if (p == buf) {
		/* Strip the comment. This must be done here so we
		 * do not acces p[-1], when p==buf.
		 */
		*p = '\0';

		p = NULL;
		continue;
	  }
	  if (p[-1]!='\\')
	    *p = '\0';
	  else {
	    char *a, *q;

	    a = p - 1;	/* Del \ chr; move rest back  */
	    q = p;
	    while (*a++ = *q++);
	  }
	}
#endif

	/* empty line ? */
	for (p = buf; isspace(*p); p++);
	if (*p == '\0') continue;

	/* compress leading spaces from continued line(s) */
	if (pos  &&  p != buf)
	  for (q = buf; (*q = *p) != '\0'; q++, p++);
	  
	/* look at last char */
	p = buf + strlen(buf) - 1;

	/* skip trailing white spaces -- we know now the line isn't empty */
	while ( isspace (*p)) p--;

	/* backslash continuation? */
	if (*p == '\\') {
	  /* compress whitespaces before \ char */
	  p--;
	  while (isspace(*p)) p--;
	  *++p = ' ';		/* replace with single space */
	  pos = (++p) - str;	/* go read more */
	  continue;
	}
	*++p = '\n';
	return FALSE;	/* a valid line is read in */
    }
}


/*
 *	Get a word from the current line, surounded by white space.
 *	return a pointer to it. String returned has no white spaces
 *	in it.
 */
char	       *
gettok(ptr)
    register char  **ptr;	/*OIS*0.80*/
{
    register char  *p;


    while (isspace(**ptr))	/* Skip spaces	*/
	(*ptr)++;

    if (**ptr == '\0')          /* Nothing after spaces  */
	return NULL;

    p = *ptr;			/* word starts here  */

    while ((**ptr != '\0') && (!isspace(**ptr)))
	(*ptr)++;		/* Find end of word  */

    *(*ptr)++ = '\0';           /* Terminate it  */

    return (p);
}
